@@ -8,6 +8,7 @@ report = {
         "architecture": None,
         "compile_time": None,
         "data": None,
+        "derivation": None,
         "description": None,
         "file_path": "C:/input_file.bin",
         "md5": "1e50210a0202497fb79bc38b6ade6c34",
@@ -20,38 +21,34 @@ report = {
     "logs": ["[+] Test info log", "[!] Test error log", "[*] Test debug log"],
     "metadata": [
         {
-            "directory_path": "C:\\windows\\temp\\1\\log",
             "file_system": None,
             "is_dir": False,
-            "name": "keydb.txt",
             "path": "C:\\windows\\temp\\1\\log\\keydb.txt",
+            "posix": False,
             "tags": [],
             "type": "path",
         },
         {
-            "directory_path": "%APPDATA%",
             "file_system": None,
             "is_dir": True,
-            "name": "foo",
             "path": "%APPDATA%\\foo",
+            "posix": False,
             "tags": [],
             "type": "path",
         },
         {
-            "directory_path": "C:\\foo",
             "file_system": None,
             "is_dir": False,
-            "name": "bar.txt",
             "path": "C:\\foo\\bar.txt",
+            "posix": False,
             "tags": [],
             "type": "path",
         },
         {
-            "directory_path": None,
             "file_system": None,
             "is_dir": False,
-            "name": "malware.exe",
-            "path": None,
+            "path": "malware.exe",
+            "posix": None,
             "tags": [],
             "type": "path",
         },
@@ -335,11 +332,10 @@ report = {
             "type": "service",
         },
         {
-            "directory_path": "%System%",
             "file_system": None,
             "is_dir": False,
-            "name": "svohost.exe",
             "path": "%System%\\svohost.exe",
+            "posix": False,
             "tags": [],
             "type": "path",
         },
@@ -354,6 +350,7 @@ report = {
             "architecture": None,
             "compile_time": None,
             "data": None,
+            "derivation": "embedded",
             "description": "Extracted backdoor Foo config file",
             "file_path": None,
             "md5": "8c41f2802904e53469390845cfeb2b28",
